//===----------------------------------------------------------------------===//
//                         DuckDB
//
// function/scalar/compressed_materialization_functions.hpp
//
//
//===----------------------------------------------------------------------===//
// This file is automatically generated by scripts/generate_functions.py
// Do not edit this file manually, your changes will be overwritten
//===----------------------------------------------------------------------===//

#pragma once

#include "duckdb/function/function_set.hpp"

namespace duckdb {

struct InternalCompressIntegralUtinyintFun {
	static constexpr const char *Name = "__internal_compress_integral_utinyint";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunctionSet GetFunctions();
};

struct InternalCompressIntegralUsmallintFun {
	static constexpr const char *Name = "__internal_compress_integral_usmallint";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunctionSet GetFunctions();
};

struct InternalCompressIntegralUintegerFun {
	static constexpr const char *Name = "__internal_compress_integral_uinteger";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunctionSet GetFunctions();
};

struct InternalCompressIntegralUbigintFun {
	static constexpr const char *Name = "__internal_compress_integral_ubigint";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunctionSet GetFunctions();
};

struct InternalCompressStringUtinyintFun {
	static constexpr const char *Name = "__internal_compress_string_utinyint";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunction GetFunction();
};

struct InternalCompressStringUsmallintFun {
	static constexpr const char *Name = "__internal_compress_string_usmallint";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunction GetFunction();
};

struct InternalCompressStringUintegerFun {
	static constexpr const char *Name = "__internal_compress_string_uinteger";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunction GetFunction();
};

struct InternalCompressStringUbigintFun {
	static constexpr const char *Name = "__internal_compress_string_ubigint";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunction GetFunction();
};

struct InternalCompressStringUhugeintFun {
	static constexpr const char *Name = "__internal_compress_string_uhugeint";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunction GetFunction();
};

struct InternalCompressStringHugeintFun {
	static constexpr const char *Name = "__internal_compress_string_hugeint";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunction GetFunction();
};

struct InternalDecompressIntegralSmallintFun {
	static constexpr const char *Name = "__internal_decompress_integral_smallint";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunctionSet GetFunctions();
};

struct InternalDecompressIntegralIntegerFun {
	static constexpr const char *Name = "__internal_decompress_integral_integer";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunctionSet GetFunctions();
};

struct InternalDecompressIntegralBigintFun {
	static constexpr const char *Name = "__internal_decompress_integral_bigint";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunctionSet GetFunctions();
};

struct InternalDecompressIntegralHugeintFun {
	static constexpr const char *Name = "__internal_decompress_integral_hugeint";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunctionSet GetFunctions();
};

struct InternalDecompressIntegralUsmallintFun {
	static constexpr const char *Name = "__internal_decompress_integral_usmallint";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunctionSet GetFunctions();
};

struct InternalDecompressIntegralUintegerFun {
	static constexpr const char *Name = "__internal_decompress_integral_uinteger";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunctionSet GetFunctions();
};

struct InternalDecompressIntegralUbigintFun {
	static constexpr const char *Name = "__internal_decompress_integral_ubigint";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunctionSet GetFunctions();
};

struct InternalDecompressIntegralUhugeintFun {
	static constexpr const char *Name = "__internal_decompress_integral_uhugeint";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunctionSet GetFunctions();
};

struct InternalDecompressStringFun {
	static constexpr const char *Name = "__internal_decompress_string";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunctionSet GetFunctions();
};

} // namespace duckdb
