#pragma once

#include <memory>

#define DUCKDB_BASE_STD

namespace duckdb_base_std {
using ::std::make_shared;
using ::std::shared_ptr;
using ::std::unique_ptr;
// using ::std::make_unique;
} // namespace duckdb_base_std
