//===----------------------------------------------------------------------===//
//                         DuckDB
//
// function/scalar/map_functions.hpp
//
//
//===----------------------------------------------------------------------===//
// This file is automatically generated by scripts/generate_functions.py
// Do not edit this file manually, your changes will be overwritten
//===----------------------------------------------------------------------===//

#pragma once

#include "duckdb/function/function_set.hpp"

namespace duckdb {

struct MapContainsFun {
	static constexpr const char *Name = "map_contains";
	static constexpr const char *Parameters = "map,key";
	static constexpr const char *Description = "Checks if a map contains a given key.";
	static constexpr const char *Example = "map_contains(MAP {'key1': 10, 'key2': 20, 'key3': 30}, 'key2')";
	static constexpr const char *Categories = "";

	static ScalarFunction GetFunction();
};

} // namespace duckdb
