#pragma once

namespace duckdb {

#ifndef DUCKDB_CLANG_TIDY
// NOLINTBEGIN
static_assert(false, "The DuckDB 'ExtensionUtil' class has been removed, see this PR for more details: "
                     "https://github.com/duckdb/duckdb/pull/17772");
// NOLINTEND
#endif

} // namespace duckdb
