#pragma once

#include <sstream>

#define DUCKDB_BASE_STD

namespace duckdb_base_std {
using ::std::basic_stringstream;
using ::std::stringstream;
using ::std::wstringstream;
} // namespace duckdb_base_std
