#include "duckdb/common/exception/binder_exception.hpp"
#include "duckdb/common/exception/catalog_exception.hpp"
#include "duckdb/common/exception/conversion_exception.hpp"
#include "duckdb/common/exception/http_exception.hpp"
#include "duckdb/common/exception/parser_exception.hpp"
#include "duckdb/common/exception/transaction_exception.hpp"
