//===----------------------------------------------------------------------===//
//                         DuckDB
//
// function/scalar/sequence_functions.hpp
//
//
//===----------------------------------------------------------------------===//
// This file is automatically generated by scripts/generate_functions.py
// Do not edit this file manually, your changes will be overwritten
//===----------------------------------------------------------------------===//

#pragma once

#include "duckdb/function/function_set.hpp"

namespace duckdb {

struct CurrvalFun {
	static constexpr const char *Name = "currval";
	static constexpr const char *Parameters = "'sequence_name'";
	static constexpr const char *Description = "Return the current value of the sequence. Note that nextval must be called at least once prior to calling currval.";
	static constexpr const char *Example = "currval('my_sequence_name')";
	static constexpr const char *Categories = "";

	static ScalarFunction GetFunction();
};

struct NextvalFun {
	static constexpr const char *Name = "nextval";
	static constexpr const char *Parameters = "'sequence_name'";
	static constexpr const char *Description = "Return the following value of the sequence.";
	static constexpr const char *Example = "nextval('my_sequence_name')";
	static constexpr const char *Categories = "";

	static ScalarFunction GetFunction();
};

} // namespace duckdb
