#include "duckdb/optimizer/rule/arithmetic_simplification.hpp"
#include "duckdb/optimizer/rule/case_simplification.hpp"
#include "duckdb/optimizer/rule/constant_order_normalization.hpp"
#include "duckdb/optimizer/rule/comparison_simplification.hpp"
#include "duckdb/optimizer/rule/conjunction_simplification.hpp"
#include "duckdb/optimizer/rule/constant_folding.hpp"
#include "duckdb/optimizer/rule/date_part_simplification.hpp"
#include "duckdb/optimizer/rule/date_trunc_simplification.hpp"
#include "duckdb/optimizer/rule/distributivity.hpp"
#include "duckdb/optimizer/rule/empty_needle_removal.hpp"
#include "duckdb/optimizer/rule/like_optimizations.hpp"
#include "duckdb/optimizer/rule/move_constants.hpp"
#include "duckdb/optimizer/rule/enum_comparison.hpp"
#include "duckdb/optimizer/rule/regex_optimizations.hpp"
#include "duckdb/optimizer/rule/ordered_aggregate_optimizer.hpp"
#include "duckdb/optimizer/rule/timestamp_comparison.hpp"
