//===----------------------------------------------------------------------===//
//                         DuckDB
//
// function/scalar/operator_functions.hpp
//
//
//===----------------------------------------------------------------------===//
// This file is automatically generated by scripts/generate_functions.py
// Do not edit this file manually, your changes will be overwritten
//===----------------------------------------------------------------------===//

#pragma once

#include "duckdb/function/function_set.hpp"

namespace duckdb {

struct OperatorAddFun {
	static constexpr const char *Name = "+";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunctionSet GetFunctions();
};

struct AddFun {
	using ALIAS = OperatorAddFun;

	static constexpr const char *Name = "add";
};

struct OperatorSubtractFun {
	static constexpr const char *Name = "-";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunctionSet GetFunctions();
};

struct SubtractFun {
	using ALIAS = OperatorSubtractFun;

	static constexpr const char *Name = "subtract";
};

struct OperatorMultiplyFun {
	static constexpr const char *Name = "*";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunctionSet GetFunctions();
};

struct MultiplyFun {
	using ALIAS = OperatorMultiplyFun;

	static constexpr const char *Name = "multiply";
};

struct OperatorFloatDivideFun {
	static constexpr const char *Name = "/";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunctionSet GetFunctions();
};

struct OperatorIntegerDivideFun {
	static constexpr const char *Name = "//";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunctionSet GetFunctions();
};

struct DivideFun {
	using ALIAS = OperatorIntegerDivideFun;

	static constexpr const char *Name = "divide";
};

struct OperatorModuloFun {
	static constexpr const char *Name = "%";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunctionSet GetFunctions();
};

struct ModFun {
	using ALIAS = OperatorModuloFun;

	static constexpr const char *Name = "mod";
};

} // namespace duckdb
