//===----------------------------------------------------------------------===//
//                         DuckDB
//
// function/scalar/system_functions.hpp
//
//
//===----------------------------------------------------------------------===//
// This file is automatically generated by scripts/generate_functions.py
// Do not edit this file manually, your changes will be overwritten
//===----------------------------------------------------------------------===//

#pragma once

#include "duckdb/function/function_set.hpp"

namespace duckdb {

struct FinalizeFun {
	static constexpr const char *Name = "finalize";
	static constexpr const char *Parameters = "col0";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunction GetFunction();
};

struct CombineFun {
	static constexpr const char *Name = "combine";
	static constexpr const char *Parameters = "col0,col1";
	static constexpr const char *Description = "";
	static constexpr const char *Example = "";
	static constexpr const char *Categories = "";

	static ScalarFunction GetFunction();
};

struct WriteLogFun {
	static constexpr const char *Name = "write_log";
	static constexpr const char *Parameters = "string";
	static constexpr const char *Description = "Writes to the logger";
	static constexpr const char *Example = "write_log('Hello')";
	static constexpr const char *Categories = "";

	static ScalarFunctionSet GetFunctions();
};

struct CurrentConnectionId {
	static constexpr const char *Name = "current_connection_id";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "Get the current connection_id";
	static constexpr const char *Example = "current_connection_id()";
	static constexpr const char *Categories = "";

	static ScalarFunction GetFunction();
};

struct CurrentTransactionId {
	static constexpr const char *Name = "current_transaction_id";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "Get the current global transaction_id";
	static constexpr const char *Example = "current_transaction_id()";
	static constexpr const char *Categories = "";

	static ScalarFunction GetFunction();
};

struct CurrentQueryId {
	static constexpr const char *Name = "current_query_id";
	static constexpr const char *Parameters = "";
	static constexpr const char *Description = "Get the current query_id";
	static constexpr const char *Example = "current_query_id()";
	static constexpr const char *Categories = "";

	static ScalarFunction GetFunction();
};

struct ParseLogMessage {
	static constexpr const char *Name = "parse_duckdb_log_message";
	static constexpr const char *Parameters = "type,message";
	static constexpr const char *Description = "Parse the message into the expected logical type";
	static constexpr const char *Example = "parse_duckdb_log_message('FileSystem', log_message)";
	static constexpr const char *Categories = "";

	static ScalarFunction GetFunction();
};

} // namespace duckdb
